import asyncio
from typing import Awaitable, List, Literal
import aiohttp

import oneai.skills as skills
from oneai.classes import *

URL = 'https://api.oneai.com/api/v0/pipeline'

api_key = ''

def process(text: str, skills: List[Skill], inputType: Literal['article', 'transcription']='article') -> List[LabeledText]:
    return asyncio.run(process_async(text, skills, inputType))

async def process_async(text: str, skills: List[Skill], inputType: Literal['article', 'transcription']='article') -> Awaitable[List[LabeledText]]:
    def build_skills(skills: List[Skill]) -> List[dict]:
        result = []
        input = 0
        for id, skill in enumerate(skills):
            result.append({
                'skill': skill.name,
                'input': input,
                'id': id + 1,
                'params': {
                    p: skill.__getattribute__(p) for p in skill._param_fields
                }
            })
            if skill.iswriting: input = id + 1
        return result


    def build_output(output_raw) -> List[LabeledText]: 
        def build_label(label_raw: dict) -> Label: return Label(
            type=label_raw.get('type', ''),
            name=label_raw.get('name', ''),
            span=label_raw.get('span', [0, 0]),
            value=label_raw.get('value', .0)
        )

        return [
            LabeledText(
                text=labeled_text['text'],
                labels=[build_label(label) for label in labeled_text['labels']]
            ) for labeled_text in output_raw
        ]

    ##############################################

    request = {
        'text': text,
        'steps': build_skills(skills),
        'include_intermediates': True,
        'input_type': inputType
    }
    headers = {
        'api-key': api_key,
        'Content-Type': 'application/json'
    }

    timeout = aiohttp.ClientTimeout(total=6000)
    async with aiohttp.ClientSession(timeout=timeout) as session:
        async with session.post(URL, headers=headers, json=request) as resp:
            if resp.status != 200:
                raise Exception # todo error types
            else:
                response = await resp.json()
                return build_output(response['output'])



from dataclasses import dataclass, field
from typing import List, Type


@dataclass
class Skill:
    name: str = ''
    iswriting: bool = False
    _param_fields: List[str] = field(default_factory=list, repr=False, init=False)

def skillclass(cls: Type=None, name: str='', iswriting: bool=False, param_fields: List[str]=[]):
    def wrap(cls):
        if not issubclass(cls, Skill):
            print(f'warning: class {cls.__name__} decorated with @skillclass does not inherit from Skill')

        def __init__(self, *args, **kwargs):
            cls_init(self, *args, **kwargs)
            Skill.__init__(self, name=name, iswriting=iswriting)
            self._param_fields = param_fields
        
        cls_init = cls.__init__
        cls.__init__ = __init__
        return cls
    
    return wrap if cls is None else wrap(cls)

@dataclass
class Label:
    type: str
    name: str
    span: List[int]
    value: float

@dataclass
class LabeledText:
    text: str
    labels: List[Label]



from dataclasses import dataclass

from oneai.classes import Skill, skillclass

@skillclass(name='enhance', iswriting=True)
class EnhanceTranscription(Skill): pass

@skillclass(name='summarize', iswriting=True, param_fields=['min_length', 'max_length'])
@dataclass
class Summarize(Skill):
    min_length: int = 5
    max_length: int = 100

@skillclass(name='emotions')
class EmotionDetection(Skill): pass

@skillclass(name='entities')
class EntityDetection(Skill): pass

@skillclass(name='keywords')
class KeywordDetection(Skill): pass

@skillclass(name='highlights')
class Highlighting(Skill): pass

@skillclass(name='sentences')
class SplitToSentences(Skill): pass
